"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _clientKinesis = require("@aws-sdk/client-kinesis");
var _nodeHttpHandler = require("@smithy/node-http-handler");
var _utilRetry = require("@smithy/util-retry");
var _bluebird = _interopRequireDefault(require("bluebird"));
var _lodash = require("lodash");
var _retry = require("../utils/retry");
var _log = require("../utils/log");
/* eslint import/no-extraneous-dependencies: ["error", {"devDependencies": true}] */

class Connector {
  constructor({
    debug,
    pipelineId,
    streamName = process.env.STREAM_NAME,
    timeout = Number(process.env.KINESIS_TIMEOUT) || Number(process.env.TIMEOUT) || 1000,
    retryConfig = _retry.defaultRetryConfig,
    additionalClientOpts = {},
    ...opt
  }) {
    this.debug = msg => debug('%j', msg);
    this.streamName = streamName || 'undefined';
    this.client = Connector.getClient(pipelineId, debug, timeout, additionalClientOpts);
    this.retryConfig = retryConfig;
    this.opt = opt;
  }
  static clients = {};
  static getClient(pipelineId, debug, timeout, additionalClientOpts) {
    const addlRequestHandlerOpts = (0, _lodash.pick)(additionalClientOpts, ['requestHandler']);
    const addlClientOpts = (0, _lodash.omit)(additionalClientOpts, ['requestHandler']);
    if (!this.clients[pipelineId]) {
      this.clients[pipelineId] = new _clientKinesis.KinesisClient({
        requestHandler: new _nodeHttpHandler.NodeHttpHandler({
          requestTimeout: timeout,
          connectionTimeout: timeout,
          ...addlRequestHandlerOpts
        }),
        retryStrategy: new _utilRetry.ConfiguredRetryStrategy(11, _retry.defaultBackoffDelay),
        logger: (0, _log.defaultDebugLogger)(debug),
        ...addlClientOpts
      });
    }
    return this.clients[pipelineId];
  }
  putRecords(inputParams) {
    const params = {
      StreamName: this.streamName,
      ...inputParams
    };
    return this._putRecords(params, []);
  }
  _putRecords(params, attempts) {
    (0, _retry.assertMaxRetries)(attempts, this.retryConfig.maxRetries);
    return (0, _retry.wait)((0, _retry.getDelay)(this.retryConfig.retryWait, attempts.length)).then(() => this._sendCommand(new _clientKinesis.PutRecordsCommand(params)).tap(this.debug).tapCatch(this.debug).then(resp => {
      if (resp.FailedRecordCount > 0) {
        return this._putRecords(unprocessed(params, resp), [...attempts, resp]);
      } else {
        return accumlate(attempts, resp);
      }
    }));
  }
  _sendCommand(command, ctx) {
    this.opt.metrics?.capture(this.client, command, 'kinesis', this.opt, ctx);
    return _bluebird.default.resolve(this.client.send(command)).tap(this.debug).tapCatch(this.debug);
  }
}
var _default = exports.default = Connector;
const unprocessed = (params, resp) => ({
  ...params,
  Records: params.Records.filter((e, i) => resp.Records[i].ErrorCode)
});
const accumlate = (attempts, resp) => attempts.reduceRight((a, c) => ({
  ...a,
  Records: [...c.Records.filter(r => !r.ErrorCode), ...a.Records.filter(r => !r.ErrorCode)],
  attempts: [...attempts, resp]
}), resp);