"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.encryptEvent = exports.encryptData = exports.decryptEvent = exports.decryptData = exports.decryptChangeEvent = void 0;
var _highland = _interopRequireDefault(require("highland"));
var _bluebird = _interopRequireDefault(require("bluebird"));
var _omit = _interopRequireDefault(require("lodash/omit"));
var _isEmpty = _interopRequireDefault(require("lodash/isEmpty"));
var _awsKmsEe = require("aws-kms-ee");
var _faults = require("./faults");
var _print = require("./print");
// -----------------------------------
// used in listeners index
// -----------------------------------

const decryptEvent = ({
  debug = (0, _print.debug)('enc'),
  prefilter = () => false,
  eemField = 'eem',
  AES = true,
  parallel = Number(process.env.ENCRYPTION_PARALLEL) || Number(process.env.PARALLEL) || 8
} = {}) => {
  const decrypt = uow => {
    if (!uow.event[eemField] || !prefilter(uow)) {
      return (0, _highland.default)(_bluebird.default.resolve(uow));
    }
    const p = _bluebird.default.resolve().then(() => {
      try {
        return (0, _awsKmsEe.decryptObject)((0, _omit.default)(uow.event, eemField), {
          ...uow.event[eemField],
          AES
        });
      } catch (err) {
        // istanbul ignore next
        return _bluebird.default.reject(err);
      }
    })
    // .tap(debug)
    .tapCatch(debug).then(decryptResponse => ({
      ...uow,
      decryptResponse,
      event: decryptResponse.object,
      undecryptedEvent: uow.event
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(p); // wrap promise in a stream
  };
  return s => s.map(decrypt).parallel(parallel);
};

// -----------------------------------
// used in trigger index
// -----------------------------------
exports.decryptEvent = decryptEvent;
const decryptChangeEvent = ({
  debug = (0, _print.debug)('enc'),
  prefilter = () => false,
  eemField = 'eem',
  AES = true,
  decryptOld = true,
  parallel = Number(process.env.ENCRYPTION_PARALLEL) || Number(process.env.PARALLEL) || 8
} = {}) => {
  const decrypt = uow => {
    if (!(uow.event.raw.new || uow.event.raw.old)[eemField] || !prefilter(uow)) {
      return (0, _highland.default)(_bluebird.default.resolve(uow));
    }
    const p = _bluebird.default.resolve().then(() => {
      try {
        return _bluebird.default.all([uow.event.raw.new ? (0, _awsKmsEe.decryptObject)((0, _omit.default)(uow.event.raw.new, eemField), {
          ...uow.event.raw.new[eemField],
          AES
        }) : _bluebird.default.resolve(undefined), decryptOld && uow.event.raw.old ? (0, _awsKmsEe.decryptObject)((0, _omit.default)(uow.event.raw.old, eemField), {
          ...uow.event.raw.old[eemField],
          AES
        }) : _bluebird.default.resolve(undefined)]);
      } catch (err) {
        // istanbul ignore next
        return _bluebird.default.reject(err);
      }
    })
    // .tap(debug)
    .tapCatch(debug).then(decryptResponse => ({
      ...uow,
      decryptResponse,
      event: {
        ...uow.event,
        raw: {
          new: decryptResponse[0]?.object,
          old: decryptResponse[1]?.object
        }
      },
      undecryptedEvent: uow.event
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(p); // wrap promise in a stream
  };
  return s => s.map(decrypt).parallel(parallel);
};

// -----------------------------------
// used in trigger pipelines prior to publish
// -----------------------------------
exports.decryptChangeEvent = decryptChangeEvent;
const encryptEvent = ({
  debug = (0, _print.debug)('enc'),
  eemField = 'eem',
  sourceField = 'event',
  targetField = 'event',
  eem,
  masterKeyAlias = process.env.MASTER_KEY_ALIAS,
  regions = process.env.KMS_REGIONS && process.env.KMS_REGIONS.split(','),
  AES = true,
  parallel = Number(process.env.ENCRYPTION_PARALLEL) || Number(process.env.PARALLEL) || 8
} = {}) => {
  const encrypt = uow => {
    if (!eem || !uow[sourceField]) {
      return (0, _highland.default)(_bluebird.default.resolve(uow));
    }
    const p = (0, _awsKmsEe.encryptObject)(uow[sourceField], {
      masterKeyAlias,
      regions,
      ...eem,
      // fields and overrides
      AES
    })
    // .tap(debug)
    .tapCatch(debug).then(encryptResponse => ({
      ...uow,
      encryptResponse,
      [targetField]: {
        ...encryptResponse.encrypted,
        [eemField]: encryptResponse.metadata
      }
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(p); // wrap promise in a stream
  };
  return s => s.map(encrypt).parallel(parallel);
};

// -------------------------------------
// used in listener pipelines prior to save
// -------------------------------------
exports.encryptEvent = encryptEvent;
const encryptData = ({
  debug = (0, _print.debug)('enc'),
  eem,
  eemField = 'eem',
  masterKeyAlias = process.env.MASTER_KEY_ALIAS,
  regions = process.env.KMS_REGIONS && process.env.KMS_REGIONS.split(','),
  AES = true
} = {}) => async data => {
  const result = await (0, _awsKmsEe.encryptObject)(data, {
    masterKeyAlias,
    regions,
    ...eem,
    // fields and overrides
    AES
  })
  // .tap(debug)
  .tapCatch(debug);
  return {
    ...result.encrypted,
    // storing the metadata with the data
    [eemField]: result.metadata
  };
};

// -----------------------------------
// used in query functions
// -----------------------------------
exports.encryptData = encryptData;
const decryptData = ({
  debug = (0, _print.debug)('enc'),
  eemField = 'eem',
  AES = true
} = {}) => async data => {
  if ((0, _isEmpty.default)(data)) return data;
  if (!data[eemField]) return data;
  const result = await (0, _awsKmsEe.decryptObject)((0, _omit.default)(data, eemField), {
    ...data[eemField],
    AES
  })
  // .tap(debug)
  .tapCatch(debug);
  return result.object;
};
exports.decryptData = decryptData;